using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;
using BearWare;

namespace ttserverlog.net
{
    class Program
    {
        static BearWare.TeamTalk4 ttclient;

        static string audio_dir = "c:\\temp";

        static int cur_cmd_id = 0;

        [STAThread] //this macro must be set in console applications for TeamTalk to work properly
        static void Main(string[] args)
        {
            ttclient = new TeamTalk4(true);

            string ipaddr = "192.168.0.55", srvpasswd = "";
            string username = "admin", password = "admin";
            int tcpport = TeamTalk4.DEFAULT_TCPPORT, udpport = TeamTalk4.DEFAULT_UDPPORT;

            Console.WriteLine("TeamTalk 4 server logger.");
            Console.WriteLine();
            Console.WriteLine("TeamTalk 4 server login information.");
            Console.Write("Specify IP-address of server to connect to: ");
            ipaddr = get_str(ipaddr);
            Console.Write("Specify TCP port: ");
            tcpport = get_int(tcpport);
            Console.Write("Specify UDP port: ");
            udpport = get_int(udpport);
            Console.Write("Specify server password: ");
            srvpasswd = get_str(srvpasswd);
            Console.WriteLine();
            Console.WriteLine("Administrator account information.");
            Console.Write("Specify username: ");
            username = get_str(username);
            Console.Write("Specify password: ");
            password = get_str(password);
            Console.Write("Specify directory where to store audio: ");
            audio_dir = get_str(audio_dir);

            if(!File.Exists("lame_enc.dll"))
                Console.WriteLine("lame_enc.dll not found, so audio will be stored to .wav instead of .mp3");

            //hook events for checking connection
            ttclient.OnConnectFailed += new TeamTalk4.Connection(ttclient_OnConnectFailed);
            ttclient.OnConnectSuccess += new TeamTalk4.Connection(ttclient_OnConnectSuccess);
            ttclient.OnConnectionLost += new TeamTalk4.Connection(ttclient_OnConnectionLost);

            //now that we got all the information we needed we can connect and logon
            if (!ttclient.Connect(ipaddr, tcpport, udpport, 0, 0))
            {
                Console.WriteLine("Failed to connect to server");
                return;
            }

            //wait for 10 seconds for connect event
            if (!ttclient.GetMessage(10000))
            {
                Console.WriteLine("Did not connect in time. Aborting...");
                return;
            }

            Debug.Assert((ttclient.Flags & ClientFlag.CLIENT_CONNECTED) == ClientFlag.CLIENT_CONNECTED);

            //hook the remaining events we want to process
            ttclient.OnCmdMyselfLoggedIn += new TeamTalk4.MyselfLoggedIn(ttclient_OnCmdMyselfLoggedIn);
            ttclient.OnCmdServerUpdate += new TeamTalk4.ServerUpdate(ttclient_OnCmdServerUpdate);
            ttclient.OnCmdChannelNew += new TeamTalk4.ChannelUpdate(ttclient_OnCmdChannelNew);
            ttclient.OnCmdChannelUpdate += new TeamTalk4.ChannelUpdate(ttclient_OnCmdChannelUpdate);
            ttclient.OnCmdChannelRemove += new TeamTalk4.ChannelUpdate(ttclient_OnCmdChannelRemove);
            ttclient.OnCmdUserLoggedIn += new TeamTalk4.UserAuth(ttclient_OnCmdUserLoggedIn);
            ttclient.OnCmdUserLoggedOut += new TeamTalk4.UserAuth(ttclient_OnCmdUserLoggedOut);
            ttclient.OnCmdUserUpdate += new TeamTalk4.UserUpdate(ttclient_OnCmdUserUpdate);
            ttclient.OnCmdUserJoinedChannel += new TeamTalk4.UserUpdate(ttclient_OnCmdUserJoinedChannel);
            ttclient.OnCmdUserLeftChannel += new TeamTalk4.UserUpdate(ttclient_OnCmdUserLeftChannel);
            ttclient.OnCmdFileNew += new TeamTalk4.FileUpdate(ttclient_OnCmdFileNew);
            ttclient.OnCmdFileRemove += new TeamTalk4.FileUpdate(ttclient_OnCmdFileRemove);
            ttclient.OnCmdUserTextMessage += new TeamTalk4.UserTextMessage(ttclient_OnCmdUserTextMessage);
            ttclient.OnCmdProcessing += new TeamTalk4.CommandProcessing(ttclient_OnCmdProcessing);
            ttclient.OnCmdError += new TeamTalk4.CommandError(ttclient_OnCmdError);
            ttclient.OnCmdSuccess += new TeamTalk4.CommandSuccess(ttclient_OnCmdSuccess);
            ttclient.OnUserAudioFile += new TeamTalk4.UserAudioFile(ttclient_OnUserAudioFile);

            //now that we're connected log on
            cur_cmd_id = ttclient.DoLogin("", srvpasswd, username, password);
            Console.WriteLine("Login command got cmd ID #" + cur_cmd_id);
            if (cur_cmd_id < 0)
            {
                Console.WriteLine("Failed to issue command. Aborting...");
                return;
            }

            //wait for login command to complete
            while (cur_cmd_id > 0)
                ttclient.GetMessage(10000);

            //if this assertion failed it's because the login information was incorrect
            Debug.Assert((ttclient.Flags & ClientFlag.CLIENT_AUTHORIZED) == ClientFlag.CLIENT_AUTHORIZED);
            //we have to be admin user to do logging on the server.
            Debug.Assert((ttclient.UserType & UserType.USERTYPE_ADMIN) == UserType.USERTYPE_ADMIN);

            //now process events forever.
            while (ttclient.GetMessage(-1)) ;
        }

        static void ttclient_OnUserAudioFile(int nUserID, AudioFileStatus nStatus)
        {
            User user;
            if (ttclient.GetUser(nUserID, out user))
            {
                switch (nStatus)
                {
                    case AudioFileStatus.AFS_ERROR :
                        Console.WriteLine("Failed to store audio file for #{0} {1}.",
                            user.nUserID, user.szNickname);
                        break;
                    case AudioFileStatus.AFS_STARTED :
                        Console.WriteLine("Storing audio from #{0} for {1} to file.",
                            user.nUserID, user.szNickname);
                        break;
                    case AudioFileStatus.AFS_FINISHED :
                        Console.WriteLine("Finished storing audio from #{0} {1} to file.",
                            user.nUserID, user.szNickname);
                        break;
                    case AudioFileStatus.AFS_ABORTED :
                        Console.WriteLine("Aborted storing audio from #{0} {1} to file.",
                            user.nUserID, user.szNickname);
                        break;
                }
            }
        }

        static void ttclient_OnCmdError(ClientError nErrorNo, int nCmdID)
        {
            Console.WriteLine("Command with ID #{0} failed with the following error: {1}",
                nCmdID, TeamTalk4.GetErrorMessage(nErrorNo));
        }

        static void ttclient_OnCmdSuccess(int nCmdID)
        {
            Console.WriteLine("Command with ID #{0} succeeded.", nCmdID);
        }

        static void ttclient_OnCmdProcessing(int nCmdID, bool bComplete)
        {
            if (!bComplete)
                Console.WriteLine("Started processing cmd ID #{0}", nCmdID);
            else
            {
                Console.WriteLine("Finished processing cmd ID #{0}", nCmdID);
                cur_cmd_id = 0;
            }
        }

        static void ttclient_OnCmdUserTextMessage(int nUserID, int nMsgID)
        {
            TextMessage textmsg;
            User user, touser;
            if (!ttclient.GetUser(nUserID, out user))
                return;
            if (!ttclient.GetTextMessage(nMsgID, true, out textmsg))
                return;
            ttclient.GetUser(textmsg.nFromUserID, out touser);
            string chanpath;
            ttclient.GetChannelPath(textmsg.nChannelID, out chanpath);
            switch (textmsg.nMsgType)
            {
                case TextMsgType.MSGTYPE_USER :
                    Console.WriteLine("Text message from user #{0} {1} to user #{2} {3} content: {4}",
                        user.nUserID, user.szNickname, touser.nUserID, touser.szNickname, textmsg.szMessage);
                    break;
                case TextMsgType.MSGTYPE_CHANNEL :
                    Console.WriteLine("Text message from user #{0} {1} to channel {2} context: {3}",
                        user.nUserID, user.szNickname, chanpath, textmsg.szMessage);
                    break;
                case TextMsgType.MSGTYPE_BROADCAST :
                    Console.WriteLine("Text messagr from user #{0} {1} to entire server content: {2}",
                        user.nUserID, user.szNickname, textmsg.szMessage);
                    break;
            }
        }

        static void ttclient_OnCmdFileRemove(int nFileID, int nChannelID)
        {
            string chanpath;
            if (ttclient.GetChannelPath(nChannelID, out chanpath))
                Console.WriteLine("File #{0} removed from channel {1}", nFileID, chanpath);
        }

        static void ttclient_OnCmdFileNew(int nFileID, int nChannelID)
        {
            BearWare.FileInfo fileinfo;
            string chanpath;
            if (ttclient.GetChannelPath(nChannelID, out chanpath) &&
                ttclient.GetChannelFileInfo(nChannelID, nFileID, out fileinfo))
                Console.WriteLine("File #{0} {1} added to channel {2}",
                    fileinfo.nFileID, fileinfo.szFileName, chanpath);

        }

        static void ttclient_OnCmdUserLeftChannel(int nUserID, int nChannelID)
        {
            User user;
            string chanpath;
            if (ttclient.GetUser(nUserID, out user) &&
                ttclient.GetChannelPath(nChannelID, out chanpath))
                Console.WriteLine("User #{0} {1} left {2}",
                    user.nUserID, user.szNickname, chanpath);
        }

        static void ttclient_OnCmdUserJoinedChannel(int nUserID, int nChannelID)
        {
            User user;
            string chanpath;
            if (ttclient.GetUser(nUserID, out user) &&
                ttclient.GetChannelPath(nChannelID, out chanpath))
            {
                Console.WriteLine("User #{0} {1} joined {2}",
                    user.nUserID, user.szNickname, chanpath);
                int cmd_id = ttclient.DoSubscribe(nUserID, Subscription.SUBSCRIBE_INTERCEPT_USER_MSG |
                    Subscription.SUBSCRIBE_INTERCEPT_CHANNEL_MSG | Subscription.SUBSCRIBE_INTERCEPT_AUDIO);
                if (cmd_id < 0)
                    Console.WriteLine("Failed to issue subscribe command");
                else
                    Console.WriteLine("Issued command to subscribe to #{0}", user.nUserID);
            }
        }

        static void ttclient_OnCmdUserUpdate(int nUserID, int nChannelID)
        {
            User user;
            string chanpath;
            if (ttclient.GetUser(nUserID, out user) &&
                ttclient.GetChannelPath(nChannelID, out chanpath))
                Console.WriteLine("User #{0} {1} updated {2}",
                    user.nUserID, user.szNickname, chanpath);
        }

        static void ttclient_OnCmdUserLoggedOut(int nUserID)
        {
            //cannot retrieve user since he no longer exists
            Console.WriteLine("User #{0} logged out", nUserID);
        }

        static void ttclient_OnCmdUserLoggedIn(int nUserID)
        {
            User user;
            if (ttclient.GetUser(nUserID, out user))
            {
                Console.WriteLine("User #{0} {1} logged in",
                    user.nUserID, user.szNickname);
                ttclient.SetUserAudioFolder(nUserID, audio_dir, "", AudioFileFormat.AFF_WAVE_FORMAT);
            }
        }

        static void ttclient_OnCmdChannelRemove(int nChannelID)
        {
            //cannot retrieve
            Console.WriteLine("Removed channel #{0}", nChannelID);
        }

        static void ttclient_OnCmdChannelUpdate(int nChannelID)
        {
            string chanpath;
            if (ttclient.GetChannelPath(nChannelID, out chanpath))
                Console.WriteLine("Updated channel {0}", chanpath);
        }

        static void ttclient_OnCmdChannelNew(int nChannelID)
        {
            string chanpath;
            if (ttclient.GetChannelPath(nChannelID, out chanpath))
            {
                Console.WriteLine("Added channel {0}. Audio storage: {1}", chanpath, audio_dir);
            }
        }

        static void ttclient_OnCmdServerUpdate()
        {
            ServerProperties srvprop;
            if (!ttclient.GetServerProperties(out srvprop))
                return;
            Console.WriteLine("Get new server properties.");
            Console.WriteLine("Server Name: " + srvprop.szServerName);
            Console.WriteLine("Server Password: " + srvprop.szServerPasswd);
            Console.WriteLine("MOTD: " + srvprop.szMOTD);
            Console.WriteLine("Server Version: " + srvprop.szServerVersion);
            if ((srvprop.uUserRights & UserRight.USERRIGHT_SUBSCRIPTIONS) != UserRight.USERRIGHT_SUBSCRIPTIONS)
            {
                Console.WriteLine("WARNING: It's not possible to modify subscriptions. Audio storage will not work.");
            }
        }

        static void ttclient_OnCmdMyselfLoggedIn(int nMyUserID)
        {
            Console.WriteLine("Logged in successfully...");
            Console.WriteLine("Got user ID #" + nMyUserID);
            Debug.Assert(nMyUserID == ttclient.UserID);
        }

        static void ttclient_OnConnectFailed()
        {
            Console.WriteLine("Failed to connect to server");
        }

        static void ttclient_OnConnectSuccess()
        {
            Console.WriteLine("Connected successfully to server");
        }

        static void ttclient_OnConnectionLost()
        {
            Console.WriteLine("Connection was lost to the server");
        }











        static string get_str(string def_str)
        {
            Console.Write("(\"" + def_str + "\")");
            string str = Console.ReadLine();
            if (str.Length == 0)
                return def_str;
            return str;
        }

        static int get_int(int def_int)
        {
            string str = get_str(def_int.ToString());
            return int.Parse(str);
        }
    }
}
